/*
 * PLOT/VIRTUAL INSTRUMENT LIBRARY
 * -------------------------------------------
 * MIGUEL TORRES TORRITI (c) 2008 v. 1.0
 * -------------------------------------------
 *
 * To run type:
 *	plot
 *
 * Description:
 *  Illustrates an implementation of basic plotting functions
 *  for data visualization from simple 2D plots to more sophisticated
 *  virtual instruments, such as heading and attitude indicators.
 *
 *  The current implementation includes:
 *   - 1D and 2D Plots
 *   - Level indicator
 *   - Analog indicator
 *   - Heading indicator (compass)
 *   - Attitude indicator (artificial horizon)
 *
 *  There exist several free and commercial 
 *  plotting/data visualization/virtual instrumentation/GUI widgets libraries.
 *  References to these libraries are given below.  However, the purpose of this
 *  library is to provide a simple to use set of basic plotting functions
 *  relying on OpenCV.  A second objective of this library is to
 *  provide an open implementation that can be studied quickly by
 *  students or people that wishes to build their own more sophisticated
 *  data visualization widgets.  An attempt has been made to make the code
 *  self-explanatory avoiding more elegant but less readable programming
 *  constructs.
 *
 * Options:
 *  -h   Prints this help.
 *  (not used) -nv  Turns verbose off.
 *  (not used) -np  Do not wait for user to press ESC when done.
 *  (not used) -nd  Do not display images.
 *  (not used) -no  Do not save the output.
 *  (not used) -e   Filename extension [=bmp]
 *  (not used) -sq initial_value final_value
 *                 Tells colorseg to process a sequence of images
 *                 starting at filename<initial_value>.ext and
 *                 ending in filename<initial_value>.ext
 *                 By default the sequence range is assumed to be between 0 and 10
 *                 with zero-padding = 3, i.e. 000, 001, ..., 010.
 *  (not used) -zp num   Pad filename suffix with num zeros.
 *  (not used) -fps num  Sets the frame acquisition rate (in frames per second).
 *                       This option is ignored in the current implementation.
 *
 * To do:
 *  1. Include ledbar display.
 *  2. Include buttons and other input widgets, such as knobs, switches, slide bars.
 *  3. Study the use of ROI and possible faster approaches for the heading and attitude 
 *     (gyro) indicators (see comments in the source code for the corresponding functions).
 *  4. Improve the 1D/2D axes plotting functions to include axes tick marks and numbers.
 *  5. Improve parameters of the plotting functions to change display colours.
 *  6. Include 3D visualization widgets.
 *  7. Clean the code/create classes.
 *
 * Author: Miguel Torres Torriti (c) 2008.02.06, v. 1.
 * Code    : 28h.dec
 * Comments:  2h.dec
 */
/* REFERENCES
 * Plotting Libraries, Virtual Instruments
 *
 * http://www.mitov.com/html/download_plotlab.html
 * http://jchart2d.sourceforge.net/
 * http://www.dislin.de/
 * http://en.wikipedia.org/wiki/List_of_graphing_software
 * http://en.wikipedia.org/wiki/Chart
 * http://www.live-graph.org/
 * http://www.oscilloscope-lib.com/
 * Open Source Glass Cockpit Project: http://www.opengc.org/index.html
 * Airbus 340 Glass Cockpit Project: http://a340gc.iradis.org/about/
 * http://www.globalmajic.com/ActiveX-Instrumentation-Library-v35-c61.html?GLOBALMA=s8bvnqi0jed47gp9lmmhv2mfk6
 * http://www.globalmajic.com/ActiveX-Aircraft-Library-v11-c79.html
 * http://www.genlogic.com/widgets.html
 * http://www.elodieroux.com/EditionsElodieRouxBonus.html
 * http://www.autoreconsystems.com/coral/util.html
 *
 * Instrument Descriptions
 *
 * http://www.fsflightschool.com/studentpilotresources.htm
 * http://www.ciamfg.com/Products.html
 *
 * Widgets
 *
 * http://en.wikipedia.org/wiki/List_of_widget_toolkits
 * http://en.wikipedia.org/wiki/Widget_toolkit
 * 
 * Google: 
 *	data visualization library scope, 
 *	real-time data plotting library,
 * 
 * Wikipedia:
 *	data plotting, labplot, virtual instrumentation, labview
 */

#include "cv.h"       // use " " instead of < > to get list of class/struct members
#include "highgui.h"  // use " " instead of < > to get list of class/struct members
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <conio.h>
#include <time.h>
#include <gsl/gsl_machine.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>

// Function prototypes
void sdata_plotlevel(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel = -1 );
void sdata_plotanalog(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel = -1 );
void sdata_plotheading(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel = -1 );
void sdata_plotgyro(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel = -1 );

// Global variables
#define pi 3.1415926535897932384626433832795

// The following images contain the plots produced by the function with the corresponding name.
// The names of the images were chosen for clarity of this exampple, however these names can be
// changed to any arbitrary name.
CvImage img_plot1d,		// 1D plot: plots variables 1..sdim versus sample number
		img_plot2d,		// 2D plot: plots variables 2..sdim versus variable 1
		img_level,		// Level plot: plots level
		img_analog,		// Analog indicator plot: plots needle
		img_heading,	// Heading indicator: plots a compass
		img_gyro,		// Attitude (gyro) indicator (aka. artificial horizon): plots pitch and roll (bank) angles
		img_ledbar,		// Led bar indicator: discrete type of level indicator 
		img_polar,		// Polar plot is a radar type plot based on the 2D plot
		img_buttons;	// Plots buttons

// Window names
char wnd_plot1d[]	= "1D Plot";
char wnd_plot2d[]	= "2D Plot";
char wnd_level[]	= "Level";
char wnd_analog[]	= "Analog Indicator";
char wnd_heading[]	= "Heading Indicator";
char wnd_gyro[]		= "Gyro Indicator";
char wnd_ledbar[]	= "Led Bar Display";
char wnd_polar[]	= "Polar Scan";
char wnd_buttons[]	= "Buttons";

// Image/Window dimensions
int plot1d_xmax = 400, plot1d_ymax = 320;
int plot2d_xmax = 400, plot2d_ymax = 320;
int level_xmax = 80, level_ymax = 320;
int analog_xmax = 320, analog_ymax = 320;
int buttons_xmax = 80, buttons_ymax = 320;
int heading_xmax = 320, heading_ymax = 320;
int gyro_xmax = 320, gyro_ymax = 320;
int ledbar_xmax = 80, ledbar_ymax = 320;
int polar_xmax = 400, polar_ymax = 400;

char tbarname[] = "Threshold";
int cr_thresh = 10;

CvPoint mpos = cvPoint(-1, -1);			// Current mouse position
CvPoint mpos_prev = cvPoint(-1, -1);	// Previous mouse position

typedef struct bbox {	// Region bounding-box coordinates
	int xmin;
	int ymin;
	int xmax;
	int ymax;
} region;

bbox rect = {0, 0, 0, 0}; // stores bounding box for user-selected regions.

struct sdata_struct {	// Samples (sequence) Data
	unsigned int Ns;	// number of samples 
 	unsigned int c;		// current pointed sample
	unsigned int sdim;	// dimension of each sample vector
	double *sd;		// samples data
} sdata = {0, 0, 3, NULL};

CvScalar color_table[18] = {	CV_RGB(255,0,0), // R
							CV_RGB(0,255,0), // G
							CV_RGB(0,0,255), // B
							CV_RGB(0,255,255), // C = G+B
							CV_RGB(255,0,255), // M = R+B
							CV_RGB(255,255,0), // Y = R+G
							CV_RGB(255,127,127), // M+Y
							CV_RGB(127,255,127), // C+Y
							CV_RGB(127,127,255), // C+M
							CV_RGB(170,255,255), // 2C+M+Y
							CV_RGB(255,170,255), // C+2M+Y
							CV_RGB(255,255,170), // C+M+2Y
							CV_RGB(255,213,213), // 2C+3M+3Y
							CV_RGB(213,255,213), // 3C+2M+3Y
							CV_RGB(213,213,213), // 3C+3M+2Y
							CV_RGB(232,255,255), // 6C+5M+5Y
							CV_RGB(255,232,255), // 5C+6M+5Y
							CV_RGB(255,255,232)  // 5C+5M+6Y
						};

struct cmdlinestruct {			// Command line data/parameters
	char basefilename[256];		// Frame base filename
	char fileextension[250];	// Frame file extension
	char filenameformat[200];	// "%s%0<n>d.%s" (filename)(zero-padding number)(extension)
	int iv;						// Frame initial value
	int fv;						// Frame final value
	double fps;					// Video writer parameters
	char filename_out[512];		// Output filename
	bool output_flag;
	bool display_flag;
	bool verbose_flag;
	bool pause_flag;
} cmdlineparam;

// Initialize command line structure
int initcmdlinestruct(cmdlinestruct* cmdlineparam)
{
	sprintf((*cmdlineparam).basefilename,"%s","ld_0398");			// Frame base filename
	sprintf((*cmdlineparam).fileextension,"%s","bmp");		// Frame file extension
	sprintf((*cmdlineparam).filenameformat,"%%s%%0%dd.%%s",3);	// Builds file suffix format string "%s%0<n>d.%s"
																// where <n> is the number for zero padding.
	(*cmdlineparam).iv=0;									// Frame initial value
	(*cmdlineparam).fv=10;									// Frame final value
	(*cmdlineparam).fps=15.0;								// Video writer parameters
	sprintf((*cmdlineparam).filename_out,"%s","output.jpg");	// Output filename
	(*cmdlineparam).output_flag=1;
	(*cmdlineparam).display_flag=1;
	(*cmdlineparam).verbose_flag=1;
	(*cmdlineparam).pause_flag=1;
	return 0;
}

// Parse command line structure
int parsecmdline(int argc, char** argv, cmdlinestruct* cmdlineparam)
{
	// Parse command line options
	for (int i=1; i<argc; i++)
	{	
		if (strcspn( argv[i], "-" )==0 && strstr(argv[i],"-nv")!=NULL)
		{
			(*cmdlineparam).verbose_flag = 0;
			break;
		}
	}
	if ((*cmdlineparam).verbose_flag) printf("Verbose is on.\n");

	for (int i=1; i<argc; i++)
	{	
		if (strcspn( argv[i], "-" )==0)
		{
			if (strstr(argv[i],"-nv")!=NULL)
			{
				(*cmdlineparam).verbose_flag = 0;
			}
			else if (strstr(argv[i],"-np")!=NULL)
			{
				(*cmdlineparam).pause_flag = 0;
			}
			else if (strstr(argv[i],"-nd")!=NULL)
			{
				if ((*cmdlineparam).verbose_flag) printf("No display.\n");
				(*cmdlineparam).display_flag = 0;
			}
			else if (strstr(argv[i],"-no")!=NULL)
			{
				if ((*cmdlineparam).verbose_flag) printf("No output.\n");
				(*cmdlineparam).output_flag = 0;
			}
			else if (strstr(argv[i],"-h")!=NULL)
			{
				printf(
					"plot -> display data visualization widgets\n"
					"\n"
					"Description:\n"
					"  Illustrates an implementation of basic plotting functions\n"
					"  for data visualization from simple 2D plots to more sophisticated\n"
					"  virtual instruments, such as heading and attitude indicators.\n"
					"\n"
					"  The current implementation includes:\n"
					"   - 1D and 2D Plots\n"
					"   - Level indicator\n"
					"   - Analog indicator\n"
					"   - Heading indicator (compass)\n"
					"   - Attitude indicator (artificial horizon)\n"
					"\n"
					"  There exist several free and commercial\n" 
					"  plotting/data visualization/virtual instrumentation/GUI widgets libraries.\n"
					"  References to these libraries are given below.  However, the purpose of this\n"
					"  library is to provide a simple to use set of basic plotting functions\n"
					"  relying on OpenCV.  A second objective of this library is to\n"
					"  provide an open implementation that can be studied quickly by\n"
					"  students or people that wishes to build their own more sophisticated\n"
					"  data visualization widgets.  An attempt has been made to make the code\n"
					"  self-explanatory avoiding more elegant but less readable programming\n"
					"  constructs.\n"
					" Options:\n"
					"  -h   Prints this help.\n"
					//"  (not used) -nv  Turns verbose off.\n"
					//"  (not used) -np  Do not wait for user to press ESC when done.\n"
					//"  (not used) -nd  Do not display images.\n"
					//"  (not used) -no  Do not save the output.\n"
					//"  (not used) -e   Filename extension [=bmp]\n"
					//"  (not used) -sq initial_value final_value\n"
					//"                 Tells colorseg to process a sequence of images\n"
					//"                 starting at filename<initial_value>.ext and\n"
					//"                 ending in filename<initial_value>.ext\n"
					//"                 By default the sequence range is assumed to be between 0 and 10\n"
					//"                 with zero-padding = 3, i.e. 000, 001, ..., 010.\n"
					//"  (not used) -zp num   Pad filename suffix with num zeros.\n"
					//"  (not used) -fps num  Sets the frame acquisition rate (in frames per second).\n"
					//"                       This option is ignored in the current implementation.\n"
					"\n"
					" To do:\n"
					"  1. Include ledbar display.\n"
					"  2. Include buttons and other input widgets, such as knobs, switches, slide\n" 
					"     bars.\n"
					"  3. Study the use of ROI and possible faster approaches for the heading and\n"
					"     attitude (gyro) indicators (see comments in the source code for the\n"
					"     corresponding functions).\n"
					"  4. Improve the 1D/2D axes plotting functions to include axes tick marks and\n"
					"     numbers.\n"
					"  5. Improve parameters of the plotting functions to change display colours.\n"
					"  6. Include 3D visualization widgets.\n"
					"  7. Clean the code/create classes.\n"
					"\n"
					" Author: Miguel Torres Torriti (c) 2008.02.06, v. 1.\n"
				);
				return -2;
			}
			else if (strstr(argv[i],"-e")!=NULL)
			{
				sprintf((*cmdlineparam).fileextension,"%s",argv[i+1]);
				i++;
			}
			else if (strstr(argv[i],"-sq")!=NULL)
			{
				(*cmdlineparam).iv=atoi(argv[i+1]);
				(*cmdlineparam).fv=atoi(argv[i+2]);
				i=i+2;
				printf("v0:%d, v1:%d\n",(*cmdlineparam).iv,(*cmdlineparam).fv);
			}
			else if (strstr(argv[i],"-zp")!=NULL)
			{
				sprintf((*cmdlineparam).filenameformat,"%%s%%0%dd.%%s",atoi(argv[i+1]));	// Builds file suffix format string "%s%0<n>d"
																							// where <n> is the number for zero padding.
				i++;
			}
			else if (strstr(argv[i],"-fps")!=NULL)
			{
				(*cmdlineparam).fps = atof(argv[i+1]);			
				i++;
			}
			else
			{
				if ((*cmdlineparam).verbose_flag) printf("Invalid option detected: %s\n", argv[i]);
			}
		}
		else if (i==1)
		{
			// Generate the output filename tokenizing the input filename
			sprintf((*cmdlineparam).basefilename, "%s", argv[i]);
			if ((*cmdlineparam).verbose_flag) printf("Input base filename: %s\n", (*cmdlineparam).basefilename);
		}
		else
		{
			sprintf((*cmdlineparam).filename_out, "%s", argv[i]);
			if ((*cmdlineparam).verbose_flag) printf("New output filename: %s\n", (*cmdlineparam).filename_out);
		}
	}

	return 0;
}

// Calculate bounding box area
double bboxarea(bbox rect)
{
	return (double)  ((rect.xmax-rect.xmin)*(rect.ymax-rect.ymin));
}

// Transform color from one coordinate system to another.
// See the OpenCV documentation for details on the color systems.
CvScalar ColorTransform(CvScalar color_in, int code)
{
	CvImage pixelcolor( cvSize(1,1), IPL_DEPTH_8U, 3);  // create a singe pixel image for color conversion
	//pixelcolor.create( img1.size(), IPL_DEPTH_8U, 3 ); // another method to create an image - from scratch
	cvSet1D( pixelcolor, 0, color_in ); //CV_RGB(color)
	cvCvtColor( pixelcolor, pixelcolor, code); //CV_BGR2YCrCb
	CvScalar color_out = cvGet1D( pixelcolor, 0 );
	return color_out;
}

// Creates an instance to store Ns samples of a sequence of variables having dimension sdim
// or sdim channles, i.e. the function allocates and initializes the memory for Ns samples
// of an sdim-dimensional vector.
void sdata_create(struct sdata_struct *sdata, unsigned int Ns, unsigned int sdim)
{
	if ((*sdata).Ns == 0)
	{
		(*sdata).c = 0;
		(*sdata).sd = (double *) malloc(Ns*sdim*sizeof(double));
		if ((*sdata).sd != NULL)
		{
			for(unsigned int dim = 0; dim < sdim; dim++)
			for(unsigned int sample = 0; sample < Ns; sample++)
				(*sdata).sd[sample+dim*((*sdata).Ns)] = 0.0;
			(*sdata).Ns = Ns;
			(*sdata).sdim = sdim;
		}
		else
		{
			(*sdata).Ns = 0;
			(*sdata).sdim = 0;
		}
	}
	else
	{
		(*sdata).sd = (double *) realloc( (*sdata).sd, Ns*sdim*sizeof(double) );
		if ((*sdata).sd != NULL)
		{
			(*sdata).Ns = Ns;
			(*sdata).sdim = sdim;
		}
		else
		{
			(*sdata).Ns = 0;
			(*sdata).sdim = 0;
		}
	}
}

// Releases the sdata structure, i.e. frees the memory and sets the 
// dimension parameters Ns and sdim to zero.
void sdata_release(struct sdata_struct *sdata)
{
	if ((*sdata).sd != NULL)
	{
		free( (*sdata).sd );
		(*sdata).c = 0;
		(*sdata).Ns = 0;
		(*sdata).sdim = 0;
	}
	else
	{
		(*sdata).c = 0;
		(*sdata).Ns = 0;
		(*sdata).sdim = 0;
	}
}

// Fills the sdata structure with some sampled sinuoids to be used as sequences of test data.
void sdata_filltest(struct sdata_struct *sdata)
{
	if ((*sdata).sd != NULL)
	{
		for (unsigned int dim = 0; dim < (*sdata).sdim; dim++)
		for (unsigned int sample = 0; sample < (*sdata).Ns; sample++)
		{
			//if (dim == 0)
			//	(*sdata).sd[sample] = (double) sample;
			//else
				(*sdata).sd[sample+dim*((*sdata).Ns)] = .5*sin(4.0*pi*((double) sample/(double) ((*sdata).Ns-1))
														  +2.0*pi*(double) (dim)/(double) ((*sdata).sdim)) + 0.5;
			printf("%d - %f\n",dim,(*sdata).sd[sample+dim*((*sdata).Ns)]);
		}
	}
}

// Display the widget.
// This function was created to facilitate displaying those images that have a
// ROI for the active part of the graph.  If cvShowImage is applied to an image
// whose ROI is set, only the ROI will be displayed, therefore it is necessary
// to reset temporarily the ROI before displaying the image.  Once the
// image has been displayed, the ROI can be set back again.
void sdata_display_widget( CvImage *img, char *wndname )
{
	CvRect ROI = cvGetImageROI( *img );
	cvResetImageROI( *img );
	cvShowImage( wndname, *img );
	cvSetImageROI( *img, ROI );
}

// Save the widget.
// This function was created to facilitate saving those images that have a
// ROI for the active part of the graph.  If cvSaveImage is applied to an image
// whose ROI is set, only the ROI will be saved, therefore it is necessary
// to reset temporarily the ROI before displaying the image.  Once the
// image has been saved, the ROI can be set back again.
void sdata_save_widget( CvImage *img, char* filename )
{
	CvRect ROI = cvGetImageROI( *img );
	cvResetImageROI( *img );
	cvSaveImage( filename, *img );
	cvSetImageROI( *img, ROI );
}

// Clear the ROI of the plot.
void sdata_plotclear(bool alias_flag, CvImage *img)
{
	double trim = 0.0, xpos, xdelta;
	CvRect ROI = cvGetImageROI( *img );
	
	if (alias_flag)
		trim = 2.0;

	cvRectangle( *img, cvPointFrom32f( cvPoint2D32f( trim, trim) ),
					   cvPointFrom32f( cvPoint2D32f( ROI.width-trim, ROI.height-trim) ),
					   CV_RGB(0,0,0), -1, CV_AA, 0);
}

// Plot axes for 1D and 2D plots.
void sdata_plot_axis(bool alias_flag, CvImage *img, char *xlabel, char *ylabel)
{
	double trim = .1;
	CvRect ROI = cvRect( trim*(*img).width(), trim*(*img).height(), (*img).width()*(1.0-2.0*trim), (*img).height()*(1.0-2.0*trim) );

	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;
	
	CvImage img_aux;
	CvImage img_text;
	CvMat *m_rot2d;

	// Create rotation matrix
	m_rot2d = cvCreateMat(2, 3, CV_32FC1);

	// Create image to store text
	cvInitFont( &font, CV_FONT_HERSHEY_PLAIN, 1.0, 1.0, 0.0, 1, CV_AA );
	sprintf(str, "%s", ylabel );
	cvGetTextSize( str, &font, &text_size, &text_baseline );
	img_text.create( cvSize(text_size.width+text_size.height, text_size.width+text_size.height), IPL_DEPTH_8U, 3);

	// --- Clear text image and set text ---
	cvSet( img_text, CV_RGB(0,0,0), NULL );
	cvPutText( img_text, str, cvPoint(img_text.width()/2-text_size.width/2, img_text.height()/2+text_size.height/2), &font, CV_RGB(255, 255, 255) );
	cv2DRotationMatrix( cvPointTo32f( cvPoint(img_text.width()/2, img_text.height()/2) ) , 90.0, 1.0, m_rot2d );

	img_aux = img_text.clone();

	cvWarpAffine( img_aux, img_text, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
		
	// --- Set translation matrix and place rotated text (Y-axis label) ---
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, ROI.x - img_text.height()/2 - (text_size.height + (*img).height()*.2*trim) );
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, (*img).height()/2-img_text.height()/2 );

	cvWarpAffine( img_text, *img, m_rot2d, CV_INTER_LINEAR, cvScalarAll(0) );

	// --- Draw axes box --- 
	cvRectangle( *img, cvPoint(ROI.x,ROI.y), cvPoint(ROI.x+ROI.width,ROI.y+ROI.height), CV_RGB(255, 255, 255), 1, CV_AA, 0 );

	// --- Set x-axis label ---
	sprintf( str, "%s", xlabel );
	cvGetTextSize( str, &font, &text_size, &text_baseline );
	cvPutText( *img, str, cvPoint((*img).width()/2-text_size.width/2, ROI.y+ROI.height+text_size.height+(*img).height()*.2*trim), &font, CV_RGB(255, 255, 255) );
	
	cvSetImageROI( *img, ROI );
}

// 1D Plot: plots variables 1..sdim versus sample number.
// Data in sdata is assumed to be scaled between 0 and 1.
void sdata_plot1d(struct sdata_struct *sdata, bool alias_flag, CvImage *img )
{
	double trim = 0.0, xpos, xdelta;
	CvRect ROI = cvGetImageROI( *img );
	
	if (alias_flag)
		trim = 2.0;

	if ((*sdata).sd != NULL)
	{
		xdelta = (double) (ROI.width - 1 - 2* (int) trim) /(double) ((*sdata).Ns-1);
		for (unsigned int dim = 0; dim < (*sdata).sdim; dim++)
		{	xpos = xdelta + trim;
			for (unsigned int sample = 1; sample < (*sdata).Ns; sample++)
			{
				cvLine( *img, cvPointFrom32f( cvPoint2D32f( xpos-xdelta, ROI.height-(trim+(*sdata).sd[sample-1+dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim)) )  ),
							  cvPointFrom32f( cvPoint2D32f( xpos,        ROI.height-(trim+(*sdata).sd[sample  +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim)) )  ),
							  CV_RGB(255, 255*dim/(*sdata).sdim,0), 1, CV_AA, 0);
				xpos = xpos + xdelta;
			}
		}
	}
}

// 2D Plot: plots variables 1..sdim versus sample number.
// Data in sdata is assumed to be scaled between 0 and 1.
void sdata_plot2d(struct sdata_struct *sdata, bool alias_flag, CvImage *img )
{
	double trim=0.0;
	CvRect ROI = cvGetImageROI( *img );
	
	if (alias_flag)
		trim = 2.0;

	if ( ((*sdata).sd != NULL) && (*sdata).sdim > 1 )
	{
		for (unsigned int dim = 1; dim < (*sdata).sdim; dim++)
		{
			for (unsigned int sample = 1; sample < (*sdata).Ns; sample++)
			{
				cvLine( *img, cvPointFrom32f( cvPoint2D32f( trim+(*sdata).sd[sample-1]*(ROI.width-1-2*(int) trim), ROI.height-(trim+(*sdata).sd[sample-1+dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim)) )  ),
							  cvPointFrom32f( cvPoint2D32f( trim+(*sdata).sd[sample  ]*(ROI.width-1-2*(int) trim), ROI.height-(trim+(*sdata).sd[sample  +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim)) )  ),
							  CV_RGB(255, 255*dim/(*sdata).sdim,0), 1, CV_AA, 0);
			}
		}
	}
}

// Dynamic 1D plot: plots variables 1..sdim versus sample number as they "arrive".
// Data in sdata is assumed to be scaled between 0 and 1.
// The current sample or data ("recently arrived" data) is indicated by 'sdata.c'.
void sdata_plot1d_dyn(struct sdata_struct *sdata, bool alias_flag, CvImage *img )
{
	double trim = 0.0, xpos, xdelta;
	unsigned int nsample, viewsamples;
	CvRect ROI = cvGetImageROI( *img );
	
	viewsamples = 50;

 	nsample = (*sdata).c % viewsamples;
	
	if (alias_flag)
		trim = 2.0;

	if ((*sdata).sd != NULL)
	{
		xdelta = (double) (ROI.width - 1 - 2* (int) trim) /(double) (viewsamples-1);
		xpos = (double) nsample*xdelta + trim;
		for (unsigned int dim = 0; dim < (*sdata).sdim; dim++)
		{	
			if ( nsample > 0 )
			{
				cvLine( *img, cvPointFrom32f( cvPoint2D32f( xpos-xdelta, ROI.height-(trim+(*sdata).sd[(*sdata).c-1+dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim)) )  ),
							  cvPointFrom32f( cvPoint2D32f( xpos       , ROI.height-(trim+(*sdata).sd[(*sdata).c  +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim)) )  ),
							  CV_RGB(255, 255*dim/(*sdata).sdim,0), 1, CV_AA, 0);
			}
			else
			{
				cvLine( *img, cvPointFrom32f( cvPoint2D32f( xpos, trim+(*sdata).sd[(*sdata).c +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim) )  ),
							  cvPointFrom32f( cvPoint2D32f( xpos, trim+(*sdata).sd[(*sdata).c +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim) )  ),
							  CV_RGB(255, 255*dim/(*sdata).sdim,0), 1, CV_AA, 0);
			}

		}
	}
}

// Plot axes for the level plot
void sdata_axislevel(bool alias_flag, CvImage *img, char *label)
{
	double trim = .1;
	CvRect ROI = cvRect( trim*(*img).width(), trim*(*img).height(), (*img).width()*(1.0-2.0*trim), (*img).height()*(1.0-2.0*trim) );

	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;

	cvRectangle( *img, cvPoint(ROI.x,ROI.y), cvPoint(ROI.x+ROI.width,ROI.y+ROI.height), CV_RGB(255, 255, 255), 1, CV_AA, 0 );

	cvInitFont( &font, CV_FONT_HERSHEY_PLAIN , 1.0, 1.0, 0, 1, CV_AA );
	sprintf(str, "%s", label);
	cvGetTextSize( str, &font, &text_size, &text_baseline );
	cvPutText( *img, str, cvPoint((*img).width()/2-text_size.width/2, ROI.y+ROI.height+text_size.height+(*img).height()*.2*trim), &font, CV_RGB(255, 255, 255) );
	
	cvSetImageROI( *img, ROI );
}

// Level plot
// Data in sdata is assumed to be scaled between 0 and 1.
void sdata_plotlevel(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel )
{
	double trim=0.0;
	CvRect ROI = cvGetImageROI( *img );
	unsigned int dim;

	if (alias_flag)
		trim = 2.0;

	if ((*sdata).sd != NULL)
	{
		if (channel == -1)
			dim = 0;
		else
			dim = channel;

		cvRectangle( *img, cvPointFrom32f( cvPoint2D32f( trim, trim) ),
					  cvPointFrom32f( cvPoint2D32f( ROI.width-trim, 
					  trim+(*sdata).sd[(*sdata).c  +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim) ) ),
					  CV_RGB(0,0,0), -1, CV_AA, 0);

		cvRectangle( *img, cvPointFrom32f( cvPoint2D32f( trim, 
						   trim+(*sdata).sd[(*sdata).c  +dim*(*sdata).Ns]*(ROI.height-1-2*(int) trim) ) ),
					  cvPointFrom32f( cvPoint2D32f( ROI.width-trim, ROI.height-trim) ),
					  CV_RGB(255,0,0), -1, CV_AA, 0);
	}
}

// Calculated the new coordinates of a rotated point in a 2D plane
CvPoint2D32f Rotate2D(CvPoint2D32f pt, double angle)
{
	CvPoint2D32f ptr;
	ptr.x =   cos(angle)*pt.x + sin(angle)*pt.y;
	ptr.y = - sin(angle)*pt.x + cos(angle)*pt.y;
	return ptr;
}

// Plot analog indicator
// Data in sdata is assumed to be scaled between 0 and 1.
void sdata_plotanalog(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel )
{
	double trim=0.0, angle=0.0;
	CvRect ROI = cvGetImageROI( *img );
	unsigned int dim;
	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;

	CvPoint2D32f ptr;

	//Use with A
	//CvPoint poly[] = { cvPoint(20,20), cvPoint(100,20), cvPoint(20,100), cvPoint(20,20) };
	//CvPoint *poly_ptr = &(poly[0]);
	//Use with B
	CvPoint polypt[3] = { cvPoint(20,20), cvPoint(100,20), cvPoint(20,100) };  // Array of vertices
	CvPoint* polypt_ptr[1]; // Array of pointers to polygon vertices
	polypt_ptr[0] = &(polypt[0]); // Set first element of the "array of pointers" to point the first polygon

	int vertices_number = 3;

	//Solution A
	//cvFillPoly( *img, &poly_ptr, &vertices_number, 1, CV_RGB(0,255,0), CV_AA, 0 );
	//Solution B
	//cvFillPoly( *img, polypt_ptr, &vertices_number, 1, CV_RGB(0,255,0), CV_AA, 0 );

	if (alias_flag)
		trim = 2.0;

	// Clear display area
	cvRectangle( *img, cvPointFrom32f( cvPoint2D32f( trim, trim) ),
					  cvPointFrom32f( cvPoint2D32f( ROI.width-trim, ROI.height-trim) ),
					  CV_RGB(0,0,32), -1, CV_AA, 0);
	
	// Fill sector
	cvEllipse( *img, cvPoint(ROI.width/2, ROI.height/2), cvSize(ROI.width/2-trim, ROI.height/2-trim), 0.0,
                -30.0, 210.0, CV_RGB(0,0,128), -1, CV_AA, 0 );

	// Fill needle base
	cvCircle( *img, cvPoint(ROI.width/2, ROI.height/2), 5, CV_RGB(255,128,0), -1, CV_AA, 0);

	// Define downward pointing arrow
	polypt_ptr[0][0].x = -5;
	polypt_ptr[0][0].y = 0;
	polypt_ptr[0][1].x = 5;
	polypt_ptr[0][1].y = 0;
	polypt_ptr[0][2].x = 0;
	polypt_ptr[0][2].y = ROI.height/2-2*trim;

	if ((*sdata).sd != NULL)
	{
		if (channel == -1)
			dim = 0;
		else
			dim = channel;

		angle = pi/3 + (*sdata).sd[(*sdata).c + dim*(*sdata).Ns]*4.0*pi/3.0;

		for( int k=0; k<vertices_number; k++ )
		{
			ptr.x = (double) polypt_ptr[0][k].x;
			ptr.y = (double) polypt_ptr[0][k].y;

			ptr = Rotate2D(ptr, angle);

			polypt_ptr[0][k].x = (int) ptr.x+ROI.width/2;
			polypt_ptr[0][k].y = (int) ptr.y+ROI.height/2;
		}

		cvFillPoly( *img, polypt_ptr, &vertices_number, 1, CV_RGB(255,128,0), CV_AA, 0 );

		sprintf(str, "%.2f", (*sdata).sd[(*sdata).c + dim*(*sdata).Ns]);
		cvInitFont( &font, CV_FONT_HERSHEY_SIMPLEX , 1.0, 1.0, 0.1, 2, CV_AA );		
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( *img, str, cvPoint(ROI.width/2-text_size.width/2, 11*ROI.height/16+text_size.height), &font, CV_RGB(255, 128, 0) );
	}
}

// Plot heading indicator axes (plots a compass "dial" with the North-South axis aligned to the screen y-axis).
// The "compass dial" is rotated by the sdata_plotheading function.
// Note that unlike other axes plotting functions (e.g. sdata_axisplot, sdata_axislevel), here the ROI of the 
// image is never set at the end.  This was needed to make the sdata_plotheading function work correctly,
// however further improvements of the code may allow to set the ROI and rotate only the part of the image
// corresponding to the "compass dial".
void sdata_axisheading(bool alias_flag, CvImage *img)
{
	double trim = .1;
	CvRect ROI = cvRect( trim*(*img).width(), trim*(*img).height(), (*img).width()*(1.0-2.0*trim), (*img).height()*(1.0-2.0*trim) );

	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;
	
	CvImage img_aux;
	CvImage img_text;
	CvMat *m_rot2d;

	cvSet( *img, CV_RGB(0,0,0), NULL );

	// Create rotation matrix
	m_rot2d = cvCreateMat(2, 3, CV_32FC1);

	// Create image to store text
	cvInitFont( &font, CV_FONT_HERSHEY_SIMPLEX , 0.7, 0.7, 0.0, 2, CV_AA );
	cvGetTextSize( "jWj", &font, &text_size, &text_baseline );
	img_text.create( cvSize(text_size.width, text_size.width), IPL_DEPTH_8U, 3);

	// --- Clear text image and set "W" ---
	cvSet( img_text, CV_RGB(0,0,0), NULL );
	cvGetTextSize( "W", &font, &text_size, &text_baseline );
	cvPutText( img_text, "W", cvPoint(img_text.width()/2-text_size.width/2, img_text.height()/2+text_size.height/2), &font, CV_RGB(0, 255, 0) );
	cv2DRotationMatrix( cvPointTo32f( cvPoint(img_text.width()/2, img_text.height()/2) ) , 90.0, 1.0, m_rot2d );

	img_aux = img_text.clone();

	cvWarpAffine( img_aux, img_text, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
		
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, ROI.x - img_text.height()/2 + 2*text_size.height+3 );
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, (*img).height()/2-img_text.height()/2 );

	cvWarpAffine( img_text, *img, m_rot2d, CV_INTER_LINEAR, cvScalarAll(0) );

	// --- Clear text image and set "S" ---
	cvSet( img_text, CV_RGB(0,0,0), NULL );
	cvGetTextSize( "S", &font, &text_size, &text_baseline );
	cvPutText( img_text, "S", cvPoint(img_text.width()/2-text_size.width/2, img_text.height()/2+text_size.height/2), &font, CV_RGB(0, 255, 0) );
	cv2DRotationMatrix( cvPointTo32f( cvPoint(img_text.width()/2, img_text.height()/2) ) , 180.0, 1.0, m_rot2d );

	img_aux = img_text.clone();

	cvWarpAffine( img_aux, img_text, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
		
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, (*img).width()/2-img_text.width()/2 );
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, ROI.height + ROI.y - img_text.height()/2 - 2*text_size.height - 3 );

	cvWarpAffine( img_text, *img, m_rot2d, CV_INTER_LINEAR, cvScalarAll(0) );

	// --- Clear text image and set "E" ---
	cvSet( img_text, CV_RGB(0,0,0), NULL );
	cvGetTextSize( "E", &font, &text_size, &text_baseline );
	cvPutText( img_text, "E", cvPoint(img_text.width()/2-text_size.width/2, img_text.height()/2+text_size.height/2), &font, CV_RGB(0, 255, 0) );
	cv2DRotationMatrix( cvPointTo32f( cvPoint(img_text.width()/2, img_text.height()/2) ) , -90.0, 1.0, m_rot2d );

	img_aux = img_text.clone();

	cvWarpAffine( img_aux, img_text, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
		
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, ROI.width + ROI.x - img_text.width()/2 - 2*text_size.height - 3);
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, (*img).height()/2-img_text.height()/2 );

	cvWarpAffine( img_text, *img, m_rot2d, CV_INTER_LINEAR, cvScalarAll(0) );

	// --- Clear text image and set "N" ---
	cvSet( img_text, CV_RGB(0,0,0), NULL );
	cvGetTextSize( "N", &font, &text_size, &text_baseline );
	cvPutText( img_text, "N", cvPoint(img_text.width()/2-text_size.width/2, img_text.height()/2+text_size.height/2), &font, CV_RGB(0, 255, 0) );
	cv2DRotationMatrix( cvPointTo32f( cvPoint(img_text.width()/2, img_text.height()/2) ) , 0.0, 1.0, m_rot2d );

	img_aux = img_text.clone();

	cvWarpAffine( img_aux, img_text, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
		
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, (*img).width()/2-img_text.width()/2 );
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, ROI.y - img_text.height()/2 + 2*text_size.height + 3 );

	cvWarpAffine( img_text, *img, m_rot2d, CV_INTER_LINEAR, cvScalarAll(0) );

	for (int k = 3; k < 36; k = k + 3)
	{
		if (k % 9 != 0)
		{
		// --- Clear text image and set k*10 degree marks ---
		cvSet( img_text, CV_RGB(0,0,0), NULL );
		sprintf(str,"%d",k);
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( img_text, str, cvPoint(img_text.width()/2-text_size.width/2, img_text.height()/2+text_size.height/2), &font, CV_RGB(0, 128, 0) );
		cv2DRotationMatrix( cvPointTo32f( cvPoint(img_text.width()/2, img_text.height()/2) ) , -(double) k *10.0, 1.0, m_rot2d );
		
		img_aux = img_text.clone();

		cvWarpAffine( img_aux, img_text, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
		
		int dx = (*img).width()/2 - img_text.width()/2;
		int dy = (*img).height()/2 - img_text.height()/2;
		int r = ROI.height/2 - 2*text_size.height - 3;

		dx = dx + (int) ((double) r*sin((double) k*10.0*pi/180.0));
		dy = dy - (int) ((double) r*cos((double) k*10.0*pi/180.0));
		cvmSet( m_rot2d, 0, 0, 1.0 );
		cvmSet( m_rot2d, 0, 1, 0.0 );
		cvmSet( m_rot2d, 0, 2, dx );
		cvmSet( m_rot2d, 1, 0, 0.0 );
		cvmSet( m_rot2d, 1, 1, 1.0 );
		cvmSet( m_rot2d, 1, 2, dy );

		cvWarpAffine( img_text, *img, m_rot2d, CV_INTER_LINEAR, cvScalarAll(0) );
		}
	}

	for (int k = 0; k < 360; k = k + 5)
	{
		int dx = (*img).width()/2;
		int dy = (*img).height()/2;
		int r = ROI.height/2 - 4;
		int dx1 = dx;
		int dy1 = dy;
		dx = dx + (int) ((double) r*sin((double) k*pi/180.0));
		dy = dy - (int) ((double) r*cos((double) k*pi/180.0));
		if (k % 10 != 0)
		{
			dx1 = dx1 + (int) ((double) (r-5)*sin((double) k*pi/180.0));
			dy1 = dy1 - (int) ((double) (r-5)*cos((double) k*pi/180.0));
			cvLine(*img, cvPointFrom32f( cvPoint2D32f(dx,dy) ), cvPointFrom32f( cvPoint2D32f(dx1,dy1) ), CV_RGB(255, 255, 255), 1, CV_AA, 0 );
		}
		else
		{
			dx1 = dx1 + (int) ((double) (r-10)*sin((double) k*pi/180.0));
			dy1 = dy1 - (int) ((double) (r-10)*cos((double) k*pi/180.0));
			cvLine(*img, cvPointFrom32f( cvPoint2D32f(dx,dy) ), cvPointFrom32f( cvPoint2D32f(dx1,dy1) ), CV_RGB(255, 255, 255), 2, CV_AA, 0 );
		}
	}

	cvReleaseMat( &m_rot2d );

	cvCircle( *img, cvPoint((*img).width()/2,(*img).height()/2), (double) ROI.width/2.0-3.0, CV_RGB(255, 255, 255), 1, CV_AA, 0 );

//	cvSetImageROI( *img, ROI );
}

// Plot a heading indicator (compass)
// For further comments see sdata_axisheading.
// Data in sdata is assumed to be scaled between 0 and 1, with 0 corresponding to 0 degrees and 1 corresponding
// to 360 degrees.
void sdata_plotheading(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel )
{
	double trim=0.0, angle=0.0;
	CvRect ROI = cvGetImageROI( *img );
	unsigned int dim;

	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;
	
	CvPoint2D32f ptr;

	CvMat *m_rot2d;
	CvImage img_aux;

	// Airplane
	CvPoint polypt1[24] = { cvPoint(0,-12),
							cvPoint(3,-10), cvPoint(3,-5), cvPoint(25,-4), cvPoint(20,0), cvPoint(3,2),
							cvPoint(3,12), cvPoint(10,12), cvPoint(8,15), cvPoint(3,16), cvPoint(2,18),
							cvPoint(1,18), cvPoint(0,20), cvPoint(-1,18),
							cvPoint(-2,18), cvPoint(-3,16), cvPoint(-8,15), cvPoint(-10,12), cvPoint(-3,12),
							cvPoint(-3,2), cvPoint(-20,0), cvPoint(-25,-4), cvPoint(-3,-5), cvPoint(-3,-10)};  // Array of vertices
	// Heading Arrow
	CvPoint polypt2[3] = { cvPoint(0,-46), cvPoint(-3,-16), cvPoint(3,-16) };
	// Heading Marker
	CvPoint polypt3[3] = { cvPoint(0, 7), cvPoint(-7,-7), cvPoint(7,-7) };

	CvPoint* polypt_ptr[2]; // Array of pointers to polygon vertices
	CvPoint *poly_ptr = &(polypt3[0]);

	polypt_ptr[0] = &(polypt1[0]); // Set first element of the "array of pointers" to point the first polygon
	polypt_ptr[1] = &(polypt2[0]);
	int vertices_number[2] = {24, 3};

	// Move plane to center of img;	
	for(int k = 0; k<vertices_number[0]; k++)
	{
		polypt1[k].x = polypt1[k].x + (*img).width()/2;
		polypt1[k].y = polypt1[k].y + (*img).height()/2;
	}

	// Define heading arrow
	polypt2[0].y = -((*img).height()/2 - .25*(*img).height() - 3);
	for(int k = 0; k<vertices_number[1]; k++)
	{
		polypt2[k].x = polypt2[k].x + (*img).width()/2;
		polypt2[k].y = polypt2[k].y + (*img).height()/2;
		polypt3[k].x = polypt3[k].x + (*img).width()/2;
		polypt3[k].y = polypt3[k].y + .1*(*img).height() + 3;
	}

	if (alias_flag)
		trim = 2.0;

	// Clear display area
	// cvRectangle( *img, cvPointFrom32f( cvPoint2D32f( trim, trim) ),
	// 				     cvPointFrom32f( cvPoint2D32f( ROI.width-trim, ROI.height-trim) ),
	//					 CV_RGB(0,0,32), -1, CV_AA, 0);
	
	if ((*sdata).sd != NULL)
	{
		if (channel == -1)
			dim = 0;
		else
			dim = channel;

		angle = (*sdata).sd[(*sdata).c + dim*(*sdata).Ns]*2.0*pi;

		m_rot2d = cvCreateMat(2, 3, CV_32FC1);

		cv2DRotationMatrix( cvPointTo32f( cvPoint(ROI.width/2, ROI.height/2) ) , angle*180.0/pi, 1.0, m_rot2d );
		
		// Redraw axis
		sdata_axisheading(1, img);

		img_aux = (*img).clone();

		cvWarpAffine( img_aux, *img, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );		

		// Draw plane and heading arrow
		cvFillPoly( *img, polypt_ptr, &(vertices_number[0]), 2, CV_RGB(255,255,255), CV_AA, 0 );
		//cvCircle( *img, cvPoint((*img).width()/2, (*img).height()/2-16), 3, CV_RGB(255,255,255), -1, CV_AA, 0);
		// Draw heading marker
		cvFillPoly( *img, &poly_ptr, &vertices_number[1], 1, CV_RGB(255,0,0), CV_AA, 0 );

		// Display heading value
		sprintf(str, "%03.1f", angle*180.0/pi);
		cvInitFont( &font, CV_FONT_HERSHEY_SIMPLEX, .7, .7, 0.1, 2, CV_AA );		
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( *img, str, cvPoint(ROI.width/2-text_size.width+30, 10*ROI.height/16+text_size.height), &font, CV_RGB(0, 255, 0) );

		cvReleaseMat( &m_rot2d );
	}
}

// Plot attitude indicator axes.
// This function plots an artificial horizon "dial" by:
// 1) Creating an image which has 3 times the height of the original image, i.e. under the assumption
//    that the original image height corresponds to a -45 to 45 deg. range (90 deg. span).and
//    that the pitch has a range of -90 to 90 deg.,
//    then the range of the "field of view" when the observer has a pitch of -90 deg. is -135 to -45, while
//    a the range of the "field of view" when the observer has a pitch of 90 deg. is 45 to 135 deg.
//    This means that the cone of the field of view can sweep a larger "cone" ranging from -135 to 135,
//    whose span is 270 deg. and therefore the observers 90 deg. field of view fits three times in the
//    larger cone.
// 2) Filling have of the upper half of the image with sky and the lower half with ground.
// 3) Applying a vertical translation (shift) proportional to the pitch to the "dial strip" (horizon dial).
// 4) Rotating the dial strip about the center of the image holding the shifted strip (not the center of the
//    dial strip itself).
// 5) Masking out the portions of the dial strip outside of the indicator region.
// The "horizon dial" is not rotated by the sdata_plotgyro function, unlike the sdata_plotheading which
// rotates the "compass dial".
// Note that unlike other axes plotting functions (e.g. sdata_axisplot, sdata_axislevel), here the ROI of the 
// image is never set at the end.  Unlike sdata_plotheadding, which needed the ROI not to be defined for it
// to work correctly, here sdata_plotgyro would not gain much but having a ROI.  Setting the ROI would affect
// the implementation of the sdata_plotgyro function and would possible require some modifications to work 
// correctly, however setting a ROI should not result in any substantial improvements.  Alternative implementations
// may result in a faster code, for example, rotating polygons and then painting, instead of painting rectangles and
// then rotating.  It is not clear if an improvement can be achieved with the polygon rotation strategy, but is
// something that should be evaluated sometime in the future.
void sdata_axisgyro(bool alias_flag, CvImage *img, double pitch, double roll)
{
	double trim = .1;
	CvRect ROI = cvRect( trim*(*img).width(), trim*(*img).height(), (*img).width()*(1.0-2.0*trim), (*img).height()*(1.0-2.0*trim) );

	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;
	
	CvImage img_aux;
	CvImage img_aux2;
	CvImage img_mask;
	
	CvMat *m_rot2d;

	img_aux.create( cvSize((*img).width(), 3*(*img).height()), IPL_DEPTH_8U, 3);
	img_aux2.create( cvSize((*img).width(), 3*(*img).height()), IPL_DEPTH_8U, 3);
	img_mask.create( cvSize((*img).width(), (*img).height()), IPL_DEPTH_8U, 1);

	// Set mask
	cvSet( img_mask, CV_RGB(0,0,0), NULL ); // reset mask
	cvCircle( img_mask, cvPoint((*img).width()/2,(*img).height()/2), (double) ROI.width/2.0-4.0, CV_RGB(255, 255, 255), -1, CV_AA, 0 );

	// Plot Horizon (Centered Sky/Ground) 
	cvRectangle( img_aux, cvPoint(0,0), cvPoint(img_aux.width(),img_aux.height()/2), CV_RGB(64,128,255), -1, CV_AA, 0 );
	cvRectangle( img_aux, cvPoint(0,img_aux.height()/2), cvPoint(img_aux.width(),img_aux.height()), CV_RGB(255,128,64), -1, CV_AA, 0 );
	cvLine( img_aux, cvPoint(0,img_aux.height()/2), cvPoint(img_aux.width(),img_aux.height()/2), CV_RGB(255,255,255), 1, CV_AA, 0 );
	
	// Draw pitch marks
	cvInitFont( &font, CV_FONT_HERSHEY_PLAIN, 1.0, 1.0, 0.0, 1, CV_AA );

	for(int k=-90; k<95; k = k+5 )
	{	// step = range/(Ns-1) -> Ns = range/step + 1;
		int ypos = (int) ((*img).height()/2.0 + ((double) k-(-90.0))/(180.0)*2.0*(double) (*img).height());
		
		if ((k % 10 == 0) && (k!=0) )
		{
			cvLine( img_aux, cvPoint(img_aux.width()/2-20,ypos), cvPoint(img_aux.width()/2+20,ypos), CV_RGB(255,255,255), 2, CV_AA, 0 );
			sprintf(str,"%2d", -k);
			cvGetTextSize( str, &font, &text_size, &text_baseline );
			cvPutText( img_aux, str, cvPoint(img_aux.width()/2+60-text_size.width/2, ypos+text_size.height/2), &font, CV_RGB(255, 255, 255) );
		}
		else
			cvLine( img_aux, cvPoint(img_aux.width()/2-10,ypos), cvPoint(img_aux.width()/2+10,ypos), CV_RGB(255,255,255), 1, CV_AA, 0 );
	}

	// Draw ground marks
	cvLine( img_aux, cvPoint(img_aux.width()/2,img_aux.height()/2), cvPoint(0,img_aux.height()/2+tan(15.0/180.0*pi)*(double) img_aux.width()/2.0), CV_RGB(255,255,255), 1, CV_AA, 0 );
	cvLine( img_aux, cvPoint(img_aux.width()/2,img_aux.height()/2), cvPoint(0,img_aux.height()/2+tan(45.0/180.0*pi)*(double) img_aux.width()/2.0), CV_RGB(255,255,255), 1, CV_AA, 0 );
	cvLine( img_aux, cvPoint(img_aux.width()/2,img_aux.height()/2), cvPoint(img_aux.width(),img_aux.height()/2+tan(15.0/180.0*pi)*(double) img_aux.width()/2.0), CV_RGB(255,255,255), 1, CV_AA, 0 );
	cvLine( img_aux, cvPoint(img_aux.width()/2,img_aux.height()/2), cvPoint(img_aux.width(),img_aux.height()/2+tan(45.0/180.0*pi)*(double) img_aux.width()/2.0), CV_RGB(255,255,255), 1, CV_AA, 0 );

	// Create rotation matrix
	m_rot2d = cvCreateMat(2, 3, CV_32FC1);

	// Define pitch translation and perform translation (vertical)
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, 0.0 );
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, (int) ((pitch-0.0)/90.0*(double) (*img).height()));

	cvWarpAffine( img_aux, img_aux2, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );

	// Draw roll marks
	for(int k=-30;k<31;k = k + 10)
	{
		int dxpos =  ((double) ROI.width/2.0-7.0)*sin((double)k*pi/180.0);
		int dypos = -((double) ROI.width/2.0-7.0)*cos((double)k*pi/180.0);
		int dxpos1 =  ((double) ROI.width/2.0-20.0)*sin((double)k*pi/180.0);
		int dypos1 = -((double) ROI.width/2.0-20.0)*cos((double)k*pi/180.0);
		cvLine( img_aux2, cvPoint(img_aux.width()/2+dxpos,img_aux.height()/2+dypos), cvPoint(img_aux.width()/2+dxpos1,img_aux.height()/2+dypos1), CV_RGB(255,255,255), 1, CV_AA, 0 );
	}

	// Define roll rotation and perform rotation (image centered)
	cv2DRotationMatrix( cvPointTo32f( cvPoint(img_aux.width()/2, (img_aux.height())/2) ) , roll, 1.0, m_rot2d );

	cvWarpAffine( img_aux2, img_aux, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );

	// Define translation and put central square from the auxiliary image into the destination image
	cvmSet( m_rot2d, 0, 0, 1.0 );
	cvmSet( m_rot2d, 0, 1, 0.0 );
	cvmSet( m_rot2d, 0, 2, 0.0 );
	cvmSet( m_rot2d, 1, 0, 0.0 );
	cvmSet( m_rot2d, 1, 1, 1.0 );
	cvmSet( m_rot2d, 1, 2, -(img_aux.height()-(*img).height())/2 );

	cvWarpAffine( img_aux, *img, m_rot2d, CV_INTER_LINEAR+CV_WARP_FILL_OUTLIERS, cvScalarAll(0) );
	
	// Mask out parts outside the dial region
	img_aux2 = (*img).clone();
	cvSet( *img, CV_RGB(0,0,0), NULL );
	cvCopy( img_aux2, *img, img_mask);

	// Draw wings
	cvLine( *img, cvPoint((*img).width()/2-0.6*((double) ROI.width/2.0-3.0),(*img).height()/2),
				  cvPoint((*img).width()/2-0.2*((double) ROI.width/2.0-3.0),(*img).height()/2), CV_RGB(255,255,0), 3, CV_AA, 0 );
	cvLine( *img, cvPoint((*img).width()/2+0.2*((double) ROI.width/2.0-3.0),(*img).height()/2),
				  cvPoint((*img).width()/2+0.6*((double) ROI.width/2.0-3.0),(*img).height()/2), CV_RGB(255,255,0), 3, CV_AA, 0 );
	cvLine( *img, cvPoint((*img).width()/2-0.2*((double) ROI.width/2.0-3.0),(*img).height()/2),
				  cvPoint((*img).width()/2,(*img).height()/2+0.2*((double) ROI.width/2.0-3.0)), CV_RGB(255,255,0), 3, CV_AA, 0 );
	cvLine( *img, cvPoint((*img).width()/2+0.2*((double) ROI.width/2.0-3.0),(*img).height()/2),
				  cvPoint((*img).width()/2,(*img).height()/2+0.2*((double) ROI.width/2.0-3.0)), CV_RGB(255,255,0), 3, CV_AA, 0 );
	cvCircle( *img, cvPoint((*img).width()/2,(*img).height()/2), 3, CV_RGB(255, 0, 0), -1, CV_AA, 0 );

	// Draw dial circle
	cvCircle( *img, cvPoint((*img).width()/2,(*img).height()/2), (double) ROI.width/2.0-3.0, CV_RGB(255, 255, 255), 1, CV_AA, 0 );

	cvReleaseMat( &m_rot2d );

//	cvSetImageROI( *img, ROI );
}

// Plot a attitude indicator (artificial horizon)
// For further comments see sdata_axisgyro.
// Data in sdata is assumed to be scaled between 0 and 1.
// If no component of the data vector is specified:
//   The first component of the data vector is treated as the pitch angle.
//   The second component of the data vector is treated as the roll angle.
// If a component is specified:
//   The chosen componet of the data vector is treated as the pitch angle.
//   The following componet of the data vector is treated as the roll angle, 
//   unless the pitch angle is the last component, in which case the last
//   but one component is used for the roll angle.
// For the pitch angle, the 0 to 1 value range of the scaled data is mapped to the range -90 to 90 degrees.
// For the roll angle, the 0 to 1 value range of the scaled data is mapped to the range -180 to 180 degrees.
void sdata_plotgyro(struct sdata_struct *sdata, bool alias_flag, CvImage *img, unsigned int channel )
{
	double trim=0.0, angle_pitch=0.0, angle_roll=0.0;
	CvRect ROI = cvGetImageROI( *img );
	unsigned int dim;

	char str[250];
	CvFont font;
	CvSize text_size;
	int text_baseline;
	
	CvImage img_aux;

	// Roll Marker
	CvPoint polypt3[3] = { cvPoint(0, 7), cvPoint(-7,-7), cvPoint(7,-7) };

	CvPoint *poly_ptr = &(polypt3[0]);

	int vertices_number = 3;

	if (alias_flag)
		trim = 2.0;

	// Clear display area
	// cvRectangle( *img, cvPointFrom32f( cvPoint2D32f( trim, trim) ),
	// 				     cvPointFrom32f( cvPoint2D32f( ROI.width-trim, ROI.height-trim) ),
	//					 CV_RGB(0,0,32), -1, CV_AA, 0);
	
	if (((*sdata).sd != NULL) && ((*sdata).sdim > 1))
	{
		if (channel == -1)
			dim = 0;
		else
			dim = channel;

		angle_pitch = ((*sdata).sd[(*sdata).c + dim*(*sdata).Ns]-.5)/.5*90.0;

		if (dim == (*sdata).sdim - 1)
			dim = dim - 1;
		else
			dim = dim + 1;

		angle_roll  = ((*sdata).sd[(*sdata).c + dim*(*sdata).Ns]-.5)/.5*180.0;
		
		// Redraw axis
		sdata_axisgyro(1, img, angle_pitch, angle_roll);

		// Draw roll marker
		for(int k = 0; k<vertices_number; k++)
		{
			polypt3[k].x = polypt3[k].x + (*img).width()/2;
			polypt3[k].y = polypt3[k].y + .1*(*img).height() + 3;
		}
		cvFillPoly( *img, &poly_ptr, &vertices_number, 1, CV_RGB(255,0,0), CV_AA, 0 );

		// Display pitch and heading value
		cvInitFont( &font, CV_FONT_HERSHEY_PLAIN, 1.0, 1.0, 0.0, 1, CV_AA );		
		
		sprintf(str, "Pitch:");
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( *img, str, cvPoint(ROI.x+10, ROI.y+ROI.height-3*text_size.height), &font, CV_RGB(0, 255, 0) );
		sprintf(str, "%-03.1f", angle_pitch);
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( *img, str, cvPoint(ROI.x+110-text_size.width, ROI.y+ROI.height-3*text_size.height), &font, CV_RGB(0, 255, 0) );
		sprintf(str, "Roll:", angle_roll);
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( *img, str, cvPoint(ROI.x+10, ROI.y+ROI.height-3*text_size.height/2), &font, CV_RGB(0, 255, 0) );		
		sprintf(str, "%-03.1f", angle_roll);
		cvGetTextSize( str, &font, &text_size, &text_baseline );
		cvPutText( *img, str, cvPoint(ROI.x+110-text_size.width, ROI.y+ROI.height-3*text_size.height/2), &font, CV_RGB(0, 255, 0) );
	}
}

void read_and_plot(struct sdata_struct *sdata, bool alias_flag,
				   CvImage *img_plot1d, CvImage *img_level, CvImage *img_analog,
				   CvImage *img_heading, CvImage *img_gyro)
{
	for (unsigned int sample = 0; sample < (*sdata).Ns; sample++)
	{
		(*sdata).c=sample;
		sdata_plot1d_dyn(sdata, alias_flag, img_plot1d);
		sdata_display_widget( img_plot1d, wnd_plot1d );
		sdata_plotlevel(sdata, alias_flag, img_level);
		sdata_display_widget( img_level, wnd_level );
		sdata_plotanalog(sdata, alias_flag, img_analog);
		sdata_display_widget( img_analog, wnd_analog );
		sdata_plotheading(sdata, alias_flag, img_heading);
		sdata_display_widget( img_heading, wnd_heading );
		sdata_plotgyro(sdata, alias_flag, img_gyro);
		sdata_display_widget( img_gyro, wnd_gyro );
		cvWaitKey(50);
	}
}

void WriteSData(struct sdata_struct *sdata, char* str)// Write Sequence Data
{   
	FILE *output = fopen( str, "w" );
	if (output != NULL)
	{
		fprintf(output, "Samples %u Channels %u\n", (*sdata).Ns, (*sdata).sdim);
		for(unsigned int sample=0; sample<(*sdata).Ns; sample++)
		{
			for(unsigned int dim=0; dim<(*sdata).sdim; dim++)
			{
				fprintf(output, "%f", (*sdata).sd[sample + dim*(*sdata).Ns] );
				if ( dim == (*sdata).sdim-1 ) fprintf(output, "\n");
				else fprintf(output," ");
			}
		}
		fclose(output);
	}
}

void ReadSData(struct sdata_struct *sdata) // Read Sequence Data
{
	FILE *input = fopen( "sdata.txt", "r" );
	unsigned int Ns, sdim;

	if ( (*sdata).sd != NULL)
		free( (*sdata).sd );

	if (input != NULL)
	{
		fscanf(input, "Samples %u Channels %u\n", &Ns, &sdim);  //&(*sdata).Ns, &(*sdata).sdim);
		(*sdata).sd = (double *) malloc(Ns*sdim*sizeof(double));
		(*sdata).Ns = Ns;
		(*sdata).sdim = sdim;
		(*sdata).c = 0;
		printf("Samples %u Channels %u\n", (*sdata).Ns, (*sdata).sdim);
		for(unsigned int sample=0; sample<(*sdata).Ns; sample++)
		{
			for(unsigned int dim=0;dim<(*sdata).sdim; dim++)
			{
				if ( dim == (*sdata).sdim-1 ) fscanf(input, "%lf\n", &(*sdata).sd[sample+dim*(*sdata).Ns] ); // Use %lf to read doubles (not %f)!!!
				else fscanf(input, "%lf", &(*sdata).sd[sample+dim*(*sdata).Ns]); // Use %lf to read doubles (not %f)!!!
				if ( dim == (*sdata).sdim-1 ) printf("%f\n", (*sdata).sd[sample+dim*(*sdata).Ns] );
				else printf("%f ", (*sdata).sd[sample+dim*(*sdata).Ns] );
			}
		}
        fclose(input);
	}
}

void on_mouse( int event, int x, int y, int flags, void* param )
{
	CvImage img_aux;

	if( img_plot1d.origin() )
		y = img_plot1d.height() - y;

	if (event == CV_EVENT_LBUTTONDOWN)
	{
		CvScalar Iv;
		mpos = cvPoint(x,y);

		Iv = cvGet2D( img_plot1d, mpos.y, mpos.x );
		printf("(%d, %d) - %d : %d : %d\n", mpos.x, mpos.y, (int) Iv.val[0], (int) Iv.val[1], (int) Iv.val[2]);
    }
	else if (event == CV_EVENT_RBUTTONUP) // || !(flags & CV_EVENT_FLAG_RBUTTON))
	{
		if (mpos.x<rect.xmin)
		{
			rect.xmax=rect.xmin;
			rect.xmin=mpos.x;
		}
		else
		{
			rect.xmax=mpos.x;
		}
		if (mpos.y<rect.ymin)
		{
			rect.ymax=rect.ymin;
			rect.ymin=mpos.y;
		}
		else
		{
			rect.ymax=mpos.y;
		}
		printf("bbox: [%d, %d]x[%d,%d]\n", rect.xmin, rect.ymin, rect.xmax, rect.ymax);

		mpos_prev = cvPoint(-1,-1);
	}
	else if (event == CV_EVENT_RBUTTONDOWN)
	{
		mpos_prev =  cvPoint(x,y);
		mpos = cvPoint(x,y);

		rect.xmin = mpos_prev.x;
		rect.ymin = mpos_prev.y;
		rect.xmax = mpos.x;
		rect.ymax = mpos.y;
	}
	else if (event == CV_EVENT_MOUSEMOVE && (flags & CV_EVENT_FLAG_RBUTTON) && !(flags & CV_EVENT_FLAG_CTRLKEY))
	{
		mpos = cvPoint(x,y);
		if (mpos_prev.x < 0)
			mpos_prev = cvPoint(x,y);

		img_aux = img_plot1d.clone();
		//cvLine( img_aux, mpos_prev, mpos, cvScalarAll(255), 5, 8, 0 );
		cvRectangle( img_aux, mpos_prev, mpos, cvScalarAll(255), 1, 8, 0 );

		//mpos_prev = mpos;
		cvShowImage( wnd_plot1d, img_aux );
	}
	else if (event == CV_EVENT_MOUSEMOVE && (flags & CV_EVENT_FLAG_RBUTTON) && (flags & CV_EVENT_FLAG_CTRLKEY))
	{
		mpos = cvPoint(x,y);
		if (mpos_prev.x < 0)
			mpos_prev = cvPoint(x,y);

		img_aux = img_plot1d.clone();
		cvLine( img_aux, mpos_prev, mpos, cvScalarAll(255), 5, 8, 0 );
		img_plot1d = img_aux.clone();

		mpos_prev = mpos;
		cvShowImage( wnd_plot1d, img_aux );
	}
}

// Define a trackbar callback
void on_trackbar(int h)
{
	sdata_plot2d( &sdata, 1, &img_plot1d );
	img_plot1d.show( wnd_plot1d );
}

int main( int argc, char** argv )
{
	char filename[512];        // Frame filename
	//int fnum=0;		           // Frame current number (not used)
	// moved to global: struct cmdlinestruct cmdlineparam;	   // Command line parameters structure
	bool hold_flag = 1;

	initcmdlinestruct( &cmdlineparam );		

	// Parse the command line parameters 
	if (parsecmdline(argc, argv, &cmdlineparam)==-2)
		return 0;

	// Print welcome message and the OpenCV version
	if (cmdlineparam.verbose_flag)
	{
		printf ("plot: Plots and virtual instruments using OpenCV version %s (%d.%d.%d)\n",
	    CV_VERSION, CV_MAJOR_VERSION, CV_MINOR_VERSION, CV_SUBMINOR_VERSION);

		printf( "Hot keys: \n"
			    "\tESC - quit the program\n"
				"\tMouse-R - select region of the 1D plot\n"
				"\tMouse-R+CTRL - draw on buffer of the 1D plot\n"
				"\tc - create memory storage for a data sequence\n"
				"\tf - fill memory storage with test data sequence\n"
				"\ts - save data sequence in memory storage to sdata.txt\n"
				"\tg - get data sequence from file sdata.txt\n"
				"\t1 - Display 1D plot: variables vs. sample number\n"
				"\t2 - Display 2D plot: variables 2..sdim vs. variable 1\n"
				"\t0 - Display data for all plots in dynamically\n"
				"\t    Simulated real-time processing with user input disable\n"
				"\th - toggle process holding (temporarily disables/enables processing)\n"
				"\t    True real-time processing with user input enable\n"
				"\tl - Display level\n"
				"\tb - blank plot and level displays\n"
				"\t< - decrease amplitude of data values by 10\%\n"
				"\t> - increase amplitude of data values by 10\%\n"
				"\t\n"
				"Type plot -h for additional help.\n" 
				"NOTE: Must create+fill (option c and f) or get (option g) data values\n"
				"      before using the display functions (options 1, 2, 0, l)\n");
	}

	// Create an image for 2D plotting and window
	img_plot1d.create( cvSize(plot1d_xmax, plot1d_ymax), IPL_DEPTH_8U, 3);
	cvSet( img_plot1d, CV_RGB(0,0,0), NULL );
	cvNamedWindow( wnd_plot1d, CV_WINDOW_AUTOSIZE );
	sdata_display_widget( &img_plot1d, wnd_plot1d );
	cvSetMouseCallback( wnd_plot1d, on_mouse, 0 );

	// Plot axes
	sdata_plot_axis(1, &img_plot1d, "X [units]", "Y [units]" );
	sdata_display_widget( &img_plot1d, wnd_plot1d );

	// Create an image for level indicator
	img_level.create( cvSize(level_xmax, level_ymax), IPL_DEPTH_8U, 3);
	cvSet( img_level, CV_RGB(0,0,0), NULL );
	cvNamedWindow( wnd_level, CV_WINDOW_AUTOSIZE );
	sdata_display_widget( &img_level, wnd_level );

	// Plot axes
	sdata_axislevel(1, &img_level, "Level" );
	sdata_display_widget( &img_level, wnd_level );

	// Create an image for analog indicator
	img_analog.create( cvSize(analog_xmax, analog_ymax), IPL_DEPTH_8U, 3);
	cvSet( img_analog, CV_RGB(0,0,0), NULL );
	cvNamedWindow( wnd_analog, CV_WINDOW_AUTOSIZE );
	sdata_display_widget( &img_analog, wnd_analog );

	// Plot axes
	sdata_plot_axis(1, &img_analog,  "Power [units]", "" );
	sdata_display_widget( &img_analog, wnd_analog );

	// Create an image for heading indicator (compass)
	img_heading.create( cvSize(heading_xmax, heading_ymax), IPL_DEPTH_8U, 3);
	cvSet( img_heading, CV_RGB(0,0,0), NULL );
	cvNamedWindow( wnd_heading, CV_WINDOW_AUTOSIZE );
	sdata_display_widget( &img_heading, wnd_heading );

	// Plot axes
	sdata_axisheading(1, &img_heading);
	sdata_display_widget( &img_heading, wnd_heading );

	// Create an image for gyro indicator (attitude indicator, roll-pitch indicator or artificial horizon)
	img_gyro.create( cvSize(gyro_xmax, gyro_ymax), IPL_DEPTH_8U, 3);
	cvSet( img_gyro, CV_RGB(0,0,0), NULL );
	cvNamedWindow( wnd_gyro, CV_WINDOW_AUTOSIZE );
	sdata_display_widget( &img_gyro, wnd_gyro );

	// Plot Axis
	sdata_axisgyro(1, &img_gyro, 0.0, 0.0);
	sdata_display_widget( &img_gyro, wnd_gyro );

	for(;;)
	{
		int c_int = cvWaitKey(10);
		if ( c_int != -1 )
		{
			unsigned char c = (unsigned char) c_int;
			if (c == 27)
			{
				break;
			}
			else if (c == 'c'|c == 'C')
			{	
				printf("Creating memory storage for data sequences...\n");
				sdata_create( &sdata, 100, 4 );
			}
			else if (c == 'r'|c == 'R')
			{	
				printf("Releasing memory storage for data sequences...\n");
				sdata_release( &sdata );
			}
			else if (c == 'f'|c == 'F')
			{	
				printf("Filling memory storage with test data sequences...\n");
				sdata_filltest( &sdata );
			}
			else if (c == '1')
			{	
				printf("Plotting 1D sequences in memory storage...\n");
				sdata_plot1d( &sdata, 1, &img_plot1d );
				sdata_display_widget( &img_plot1d, wnd_plot1d );
			}
			else if (c == '2')
			{	
				printf("XY Plotting 2D sequences in memory storage...\n");
				sdata_plot2d( &sdata, 1, &img_plot1d );
				sdata_display_widget( &img_plot1d, wnd_plot1d );
			}
			else if (c == '0')
			{	
				printf("Plotting 1D sequences in memory storage in real-time...\n");
				//BlankClass( &cpar );
				read_and_plot( &sdata, 1, &img_plot1d, &img_level, &img_analog, &img_heading, &img_gyro );
			}
			else if (c == 'l'|c == 'L')
			{	
				printf("Plotting level...\n");
				sdata_plotlevel( &sdata, 1, &img_level );
				sdata_display_widget( &img_level, wnd_level );
			}
			else if (c == 'b'|c == 'B')
			{	
				printf("Clearing plots...\n");
				sdata_plotclear( 1, &img_plot1d );
				sdata_display_widget( &img_plot1d, wnd_plot1d );
				sdata_plotclear( 1, &img_level );
				sdata_display_widget( &img_level, wnd_level );
			}
			else if (c == '>')
			{	
				printf("Incrementing samples amplitudes by 10%...\n");
				if (sdata.sd != NULL)
				{	
					for(unsigned int sample=0; sample<sdata.Ns; sample++)
					for(unsigned int dim=0; dim<sdata.sdim; dim++)
						sdata.sd[sample+dim*sdata.Ns] = 1.1*sdata.sd[sample+dim*sdata.Ns]-.05;
				}
			}
			else if (c == '<')
			{	
				printf("Decrementing samples amplitudes by 10%...\n");
				if (sdata.sd != NULL)
				{	
					for(unsigned int sample=0; sample<sdata.Ns; sample++)
					for(unsigned int dim=0; dim<sdata.sdim; dim++)
						sdata.sd[sample+dim*sdata.Ns] = .9*sdata.sd[sample+dim*sdata.Ns]+.05;
				}
			}
			else if (c == 'h'|c == 'H')
			{	
				printf("Turning off process holding ...\n");
				hold_flag = !hold_flag;
			}
			else if (c == 'g'|c == 'G')
			{	
				printf("Reading sequence data...\n");
				ReadSData( &sdata );
			}
			else if (c == 's'|c == 'S')
			{	
				printf("Saving sequence data...\n");
				WriteSData( &sdata, "sdata.txt" );
				sdata_save_widget( &img_plot1d, "plot.jpg" );
			}
			else
				printf("%c = %d\n", c, c);
		}
		if (!hold_flag)
		{	
			if (sdata.sd == NULL)
			{	
				sdata_create( &sdata, 100, 4 );
				sdata_filltest( &sdata );
			}
			sdata_plot1d_dyn( &sdata, 1, &img_plot1d);
			sdata_display_widget( &img_plot1d, wnd_plot1d );
			sdata_plotlevel( &sdata, 1, &img_level);
			sdata_display_widget( &img_level, wnd_level );
			sdata_plotanalog( &sdata, 1, &img_analog);
			sdata_display_widget( &img_analog, wnd_analog );
			sdata_plotheading( &sdata, 1, &img_heading);
			sdata_display_widget( &img_heading, wnd_heading );
			sdata_plotgyro( &sdata, 1, &img_gyro);
			sdata_display_widget( &img_gyro, wnd_gyro );
			sdata.c++;
			if ( sdata.c >= sdata.Ns )
			{
				sdata.c = 0;
			}
			cvWaitKey(40);
		}
	} //for

	cvDestroyAllWindows();	
    return 0;
    // all the images will be released automatically
}
